
# indigoPlugin	- a Python scripted shader	for Maya
#
# While this shader doesn't really do anything special
# in Maya itself,	it's a convenient	way to present	the
# parameters needed for	Indigo materials to the	user,	
# and	to	pick up those parameters in the MtI	exporter.
#
# (c)	Doug Hammond 2008
# THE	SOFTWARE	IS	PROVIDED	"AS IS",	WITHOUT WARRANTY OF ANY	KIND.	
# THE	AUTHOR DOES	NOT MAKE	AND HEREBY DISCLAIMS	ANY EXPRESS	OR	IMPLIED 
# WARRANTIES INCLUDING,	BUT NOT LIMITED TO, THE	WARRANTIES OF 
# NON-INFRINGEMENT, MERCHANTABILITY	OR	FITNESS FOR	A PARTICULAR 
# PURPOSE, OR ARISING FROM	A COURSE	OF	DEALING,	USAGE, OR 
# TRADE PRACTICE.	IN	NO	EVENT	WILL THE	AUTHOR
# BE LIABLE	FOR ANY LOST REVENUES, DATA, OR PROFITS, OR SPECIAL, 
# DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES, EVEN	IF	THE AUTHOR 
# HAS	BEEN ADVISED OF THE POSSIBILITY OR PROBABILITY OF SUCH DAMAGES.
#
# $Id: indigoPlugin.py 208	2008-03-22 13:14:12Z	doug $
#+

import sys,	math
import maya.OpenMaya	as	OpenMaya
import maya.OpenMayaRender	as	OpenMayaRender
import maya.OpenMayaUI as OpenMayaUI
import maya.OpenMayaMPx	as	OpenMayaMPx
import maya.cmds	as	cmds

version = cmds.about(api=True)

# ------------------------------------------------------------------------------

# initialize the script	plug-ins
def initializePlugin(mobject):
	mplugin	= OpenMayaMPx.MFnPlugin(mobject)
	try:
		# indigoShader
		mplugin.registerNode(	indigoShader.nodeName(),
										indigoShader.nodeId(),
										indigoShader.nodeCreator,
										indigoShader.nodeInitializer,
										OpenMayaMPx.MPxNode.kDependNode,
										indigoShader.nodeClassify() )
		# indigoSpecular
#		mplugin.registerNode(	indigoSpecular.nodeName(),
#										indigoSpecular.nodeId(),
#										indigoSpecular.nodeCreator,
#										indigoSpecular.nodeInitializer,
#										OpenMayaMPx.MPxNode.kDependNode,
#										indigoSpecular.nodeClassify() )
					
		# indigoExitPortalLocator
		mplugin.registerNode(	indigoExitPortalLocator.nodeName(),
										indigoExitPortalLocator.nodeId(),
										indigoExitPortalLocator.nodeCreator,
										indigoExitPortalLocator.nodeInitializer,
										OpenMayaMPx.MPxNode.kLocatorNode	)
#			indigoText
		mplugin.registerNode(	indigoText.nodeName(),
										indigoText.nodeId(),
										indigoText.nodeCreator,
										indigoText.nodeInitializer,
										OpenMayaMPx.MPxNode.kDependNode,
										indigoText.nodeClassify() )
		# indigoSpectrum
		mplugin.registerNode(	indigoSpectrum.nodeName(),
										indigoSpectrum.nodeId(),
										indigoSpectrum.nodeCreator,
										indigoSpectrum.nodeInitializer,
										OpenMayaMPx.MPxNode.kDependNode,
										indigoSpectrum.nodeClassify() )
		# indigoMedium
		mplugin.registerNode(	indigoMedium.nodeName(),
										indigoMedium.nodeId(),
										indigoMedium.nodeCreator,
										indigoMedium.nodeInitializer,
										OpenMayaMPx.MPxNode.kDependNode,
										indigoMedium.nodeClassify() )
										
	except:
		sys.stderr.write( "Failed to register Indigo Plugin")
		raise

# uninitialize	the script plug-in
def uninitializePlugin(mobject):
	
	mplugin	= OpenMayaMPx.MFnPlugin(mobject)
	
	try:
		mplugin.deregisterNode( indigoShader.nodeId()	)
#		mplugin.deregisterNode( indigoSpecular.nodeId()	)
		mplugin.deregisterNode( indigoExitPortalLocator.nodeId() )
		mplugin.deregisterNode( indigoText.nodeId() )
		mplugin.deregisterNode( indigoSpectrum.nodeId()	)
		mplugin.deregisterNode( indigoMedium.nodeId()	)
		
	except:
		sys.stderr.write( "Failed to deregister	Indigo Plugin"	)
		raise

# ------------------------------------------------------------------------------

class	indigoExitPortalLocator(OpenMayaMPx.MPxLocatorNode):

	@staticmethod
	def nodeName():
		return	"indigoExitPortal"

	@staticmethod
	def nodeId():
		return	OpenMaya.MTypeId(0x6D746902)
	
	@staticmethod
	def nodeInitializer():
		return	OpenMaya.MStatus.kSuccess

	@staticmethod
	def nodeCreator():
		return	indigoExitPortalLocator()
						
	def __init__(self):
		OpenMayaMPx.MPxLocatorNode.__init__(self)
		
		glRenderer =	OpenMayaRender.MHardwareRenderer.theRenderer()
		self.glFT	= glRenderer.glFunctionTable()	
		
	@staticmethod		
	def isBounded():
		return	True;
		
	def boundingBox(self):
		bbox =	OpenMaya.MBoundingBox()
		bbox.expand(	OpenMaya.MPoint( -0.5,	0.0, -0.5 )	)
		bbox.expand(	OpenMaya.MPoint(	0.5,	0.0, -0.5 )	)
		bbox.expand(	OpenMaya.MPoint(	0.5,	0.0,	0.5 )	)
		bbox.expand(	OpenMaya.MPoint( -0.5,	0.0,	0.5 )	)
		bbox.expand(	OpenMaya.MPoint(	0.0, -0.5,	0.0 )	)
		bbox.expand(	OpenMaya.MPoint(	0.0,	0.0,	0.0 )	)
		return	bbox
		
	def draw(self, view, DGpath,	style, status):
		
		try:
				col =	self.colorRGB(	status )
				
				view.beginGL()
				self.glFT.glPushAttrib(	OpenMayaRender.MGL_CURRENT_BIT )
				
				self.glFT.glColor3f(	1,	0,	0 )
				self.glFT.glBegin( OpenMayaRender.MGL_LINES )
				self.glFT.glVertex3f( 0.0,	-0.5,	0.0 )
				self.glFT.glVertex3f( 0.0,	0.0,	0.0 )
				self.glFT.glEnd()
				
				self.glFT.glEnable( OpenMayaRender.MGL_BLEND	)
				self.glFT.glBlendFunc( OpenMayaRender.MGL_SRC_ALPHA, OpenMayaRender.MGL_ONE_MINUS_SRC_ALPHA )
				
				if	status == OpenMayaUI.M3dView.kLead:
					self.glFT.glColor4f( 0, 1, 0, 0.3 )
				else:
					self.glFT.glColor4f( col.r, col.g,	col.b, 0.3 )
				
				self.glFT.glBegin( OpenMayaRender.MGL_QUADS )
				self.glFT.glVertex3f( -0.5, 0.0,	-0.5 )
				self.glFT.glVertex3f(0.5, 0.0,	-0.5 )
				self.glFT.glVertex3f(0.5, 0.0,	0.5 )
				self.glFT.glVertex3f( -0.5, 0.0,	0.5 )
				self.glFT.glEnd()
				self.glFT.glDisable(	OpenMayaRender.MGL_BLEND )
				
				self.glFT.glPopAttrib()
				view.endGL()
		except:
				sys.stderr.write("Failed to draw	indigoExitPortal\n")
				raise
				
				
# ------------------------------------------------------------------------------

class	indigoSpectrum(OpenMayaMPx.MPxNode):

	@staticmethod
	def nodeName():
		return	"indigoSpectrum"

	@staticmethod
	def nodeId():
		return	OpenMaya.MTypeId(0x6D746A01)
		
	@staticmethod
	def nodeCreator():
		return	OpenMayaMPx.asMPxPtr( indigoSpectrum() )
				
	@staticmethod
	def nodeClassify():
		if version < 201100:
			return	"utility/color"
		else:
			return "utility/igo"
			
	def __init__(self):
		OpenMayaMPx.MPxNode.__init__(self)
		aOutColor = OpenMaya.MObject()
		aOutAlpha = OpenMaya.MObject()
		spectrumType = OpenMaya.MObject()
		
		# peak	attributes
		peak_min = OpenMaya.MObject()
		peak_width =	OpenMaya.MObject()
		base_value =	OpenMaya.MObject()
		peak_value =	OpenMaya.MObject()
		
		# blackbody attribute
		temperature = OpenMaya.MObject()
		gain =	OpenMaya.MObject()
		
		# uniform	attributes
		value = OpenMaya.MObject()
		
		# regular	tabulated
		startWavelength =	OpenMaya.MObject()
		endWavelength = OpenMaya.MObject()
		valArray = OpenMaya.MObject()
	
	@staticmethod
	def nodeInitializer():
		nAttr			=	OpenMaya.MFnNumericAttribute()
		lAttr			=	OpenMaya.MFnLightDataAttribute()
		enumAttr		= OpenMaya.MFnEnumAttribute()
		tAttr			= OpenMaya.MFnTypedAttribute()
		gAttr			= OpenMaya.MFnGenericAttribute()
		sAttr			= OpenMaya.MFnStringData()
		
		try:
			
				kDefaultStringAttrValue = "0 1 0"
				stringData = sAttr.create(kDefaultStringAttrValue)
		
				indigoSpectrum.aOutColor	= nAttr.createColor("outColor", "oc")
				makeOutput(nAttr)
		
				indigoSpectrum.aOutAlpha	= nAttr.create("outAlpha", "", OpenMaya.MFnNumericData.kFloat)
				makeOutput(nAttr)
				
				indigoSpectrum.spectrumType = enumAttr.create("spectrum", "spc",		0)
				#	PEAK
				enumAttr.addField( "blackbody", 0)
				enumAttr.addField( "uniform", 1)
				enumAttr.addField( "peak", 2)
				enumAttr.addField( "regular tabulated",	3)
				makeInput(nAttr)

				indigoSpectrum.peak_min =	nAttr.create("peak_min", "pkmn",	OpenMaya.MFnNumericData.kFloat, 380.00)
				makeInput(nAttr)
				indigoSpectrum.peak_width	= nAttr.create("peak_width", "pkwd", OpenMaya.MFnNumericData.kFloat,	1.0)
				makeInput(nAttr)
				indigoSpectrum.base_value	= nAttr.create("base_value", "pkbv", OpenMaya.MFnNumericData.kFloat,	1.0)
				makeInput(nAttr)
				indigoSpectrum.peak_value	= nAttr.create("peak_value", "pkpv", OpenMaya.MFnNumericData.kFloat,	0.0)
				makeInput(nAttr)
				
				indigoSpectrum.temperature =	nAttr.create("temperature", "t",	OpenMaya.MFnNumericData.kFloat, 6500.0)
				makeInput(nAttr)
				indigoSpectrum.gain	=	nAttr.create("gain", "g",	OpenMaya.MFnNumericData.kFloat, 0.01)
				makeInput(nAttr)
				
				indigoSpectrum.value =	nAttr.create("value","uv",	OpenMaya.MFnNumericData.kFloat, 0.5)
				makeInput(nAttr)
				
				indigoSpectrum.startWavelength= nAttr.create("startWavelength","sw", OpenMaya.MFnNumericData.kFloat, 400.0)
				nAttr.setMin(0.0)
				nAttr.setMax(400.0)
				makeInput(nAttr)
				indigoSpectrum.endWavelength= nAttr.create("endWavelength","ew", OpenMaya.MFnNumericData.kFloat, 700.0)
				nAttr.setMin(700.0)
				makeInput(nAttr)
				indigoSpectrum.values = tAttr.create("values", "v", OpenMaya.MFnData.kString, stringData)
				makeInput(tAttr)
				
		except:
				sys.stderr.write("Failed to create attributes in 'indigoSpectrum'\n")
				raise
				
		try:
				indigoSpectrum.addAttribute(indigoSpectrum.aOutColor)
				indigoSpectrum.addAttribute(indigoSpectrum.aOutAlpha)
				indigoSpectrum.addAttribute(indigoSpectrum.spectrumType)
				
				#uniform
				indigoSpectrum.addAttribute(indigoSpectrum.value)
				
				#blackbody
				indigoSpectrum.addAttribute(indigoSpectrum.temperature)
				indigoSpectrum.addAttribute(indigoSpectrum.gain)

				#peak	attrs
				indigoSpectrum.addAttribute(indigoSpectrum.peak_min)
				indigoSpectrum.addAttribute(indigoSpectrum.peak_width)
				indigoSpectrum.addAttribute(indigoSpectrum.base_value)
				indigoSpectrum.addAttribute(indigoSpectrum.peak_value)
								
				#tabulated data
				indigoSpectrum.addAttribute(indigoSpectrum.startWavelength)
				indigoSpectrum.addAttribute(indigoSpectrum.endWavelength)
#				indigoSpectrum.addAttribute(indigoSpectrum.numValues)
				
				indigoSpectrum.addAttribute(indigoSpectrum.values)
				
		except:
				sys.stderr.write("Failed to add attributes in indigoSpectrum\n")
				raise
	
#			try:
#				#	Diffuse Material
##				indigoSpectrum.attributeAffects (indigoSpectrum.rgb,			indigoSpectrum.aOutColor)
#	#		indigoSpectrum.attributeAffects (indigoSpectrum.common_BumpMapping,		indigoSpectrum.aOutColor)
#				
#			except:
#				sys.stderr.write("Failed in setting attributeAffects	in	indigoSpectrum\n")
#				raise
				
# ------------------------------------------------------------------------------

class	indigoMedium(OpenMayaMPx.MPxNode):

	@staticmethod
	def nodeName():
		return	"indigoMedium"

	@staticmethod
	def nodeId():
		return	OpenMaya.MTypeId(0x6D746A02)
		
	@staticmethod
	def nodeCreator():
		return	OpenMayaMPx.asMPxPtr( indigoMedium() )
				
	@staticmethod
	def nodeClassify():
		if version < 201100:
			return	"utility/color"
		else:
			return "utility/igo"

	def __init__(self):
		
		OpenMayaMPx.MPxNode.__init__(self)
		
		precedence = OpenMaya.MObject()	#	int
		mediumType = OpenMaya.MObject()	#	int
		
		# basic
		ior = OpenMaya.MObject()	#	float
		cauchy_b_coeff = OpenMaya.MObject()	#	float
		absorption_coefficient_spectrum = OpenMaya.MObject()	#	spectrum
		subsurface_scattering = OpenMaya.MObject()	#	bool
		scattering_coefficient_spectrum = OpenMaya.MObject()	#	spectrum
		uniform_phf = OpenMaya.MObject()	#	uniform phase function
		phase_function = OpenMaya.MObject()	#	phase function
		
		# epidermis
		melanin_fraction = OpenMaya.MObject()	#	float
		melanin_type_blend = OpenMaya.MObject()	#	float
		
		# dermis
		hemoglobin_fraction = OpenMaya.MObject()	#	float
		
		# atmosphere
		center = OpenMaya.MObject()	#	point
		turbidity = OpenMaya.MObject()	#	float
	
	@staticmethod
	def nodeInitializer():
		nAttr			=	OpenMaya.MFnNumericAttribute()
		lAttr			=	OpenMaya.MFnLightDataAttribute()
		enumAttr		= OpenMaya.MFnEnumAttribute()
		tAttr			= OpenMaya.MFnTypedAttribute()
		gAttr			= OpenMaya.MFnGenericAttribute()
		sAttr			= OpenMaya.MFnStringData()
		
		try:
				indigoMedium.outColor	= nAttr.createColor("outColor", "oc")
				nAttr.setKeyable(0)
				nAttr.setStorable(0)
				nAttr.setReadable(1)
				nAttr.setWritable(0)
				
				indigoMedium.mediumType = enumAttr.create("medium", "med", 0)
				enumAttr.addField( "basic", 0)
				enumAttr.addField( "epidermis", 1)
				enumAttr.addField( "dermis", 2)
				enumAttr.addField( "atmosphere",	3)
				
				indigoMedium.precedence = nAttr.create("precedence","pre",	OpenMaya.MFnNumericData.kInt, 10)

				# basic
				indigoMedium.ior =	nAttr.create("ior", "ior",	OpenMaya.MFnNumericData.kFloat, 1.33)
				nAttr.setMin(1.0)
				indigoMedium.cauchy_b_coeff =	nAttr.create("cauchyB", "cau",	OpenMaya.MFnNumericData.kFloat, 0.0)
				nAttr.setMin(0.0)
				indigoMedium.absorption_coefficient_spectrum= nAttr.createColor("absorbtion","abs")
				indigoMedium.inverseAbsorbtion	= nAttr.create("inverseAbsorbtion", "iab", OpenMaya.MFnNumericData.kBoolean)
				indigoMedium.subsurface_scattering	= nAttr.create("subsurface_scattering", "sss", OpenMaya.MFnNumericData.kBoolean)
				indigoMedium.scattering_coefficient_spectrum= nAttr.createColor("scattering","sct")
				indigoMedium.uniform_phf	= nAttr.create("uniformPhaseFunction", "upf", OpenMaya.MFnNumericData.kBoolean, True)
				indigoMedium.phase_function =	nAttr.createColor("phaseFunction", "phf")
		
				# epidermis
				indigoMedium.melanin_fraction = nAttr.create("melaninFraction", "mfr",	OpenMaya.MFnNumericData.kFloat, 0.0)
				indigoMedium.melanin_type_blend = nAttr.create("melaninTypeBlend", "mtb",	OpenMaya.MFnNumericData.kFloat, 0.0)
				
				# dermis
				indigoMedium.hemoglobin_fraction = nAttr.create("hemoglobinFraction", "hef",	OpenMaya.MFnNumericData.kFloat, 0.0)
				
				# atmosphere
				indigoMedium.turbidity = nAttr.create("turbidity", "tur",	OpenMaya.MFnNumericData.kFloat, 1.0)
				indigoMedium.center	=	nAttr.createPoint("center", "cen")
				
		except:
				sys.stderr.write("Failed to create attributes in 'indigoMedium'\n")
				raise
				
		try:
				indigoMedium.addAttribute(indigoMedium.outColor)
				indigoMedium.addAttribute(indigoMedium.mediumType)
				indigoMedium.addAttribute(indigoMedium.precedence)
				
				indigoMedium.addAttribute(indigoMedium.ior)
				indigoMedium.addAttribute(indigoMedium.cauchy_b_coeff)
				indigoMedium.addAttribute(indigoMedium.absorption_coefficient_spectrum)
				indigoMedium.addAttribute(indigoMedium.inverseAbsorbtion)
				indigoMedium.addAttribute(indigoMedium.subsurface_scattering)
				indigoMedium.addAttribute(indigoMedium.scattering_coefficient_spectrum)
				indigoMedium.addAttribute(indigoMedium.uniform_phf)
				indigoMedium.addAttribute(indigoMedium.phase_function)
				
				indigoMedium.addAttribute(indigoMedium.melanin_fraction)
				indigoMedium.addAttribute(indigoMedium.melanin_type_blend)
				
				indigoMedium.addAttribute(indigoMedium.hemoglobin_fraction)
				
				indigoMedium.addAttribute(indigoMedium.center)
				indigoMedium.addAttribute(indigoMedium.turbidity)
				
				
		except:
				sys.stderr.write("Failed to add attributes in indigoMedium\n")
				raise
				
		
#-------------------------------------------------------------------------------

class	indigoText(OpenMayaMPx.MPxNode):
	
	@staticmethod
	def nodeName():
		return	"indigoText"
	
	@staticmethod
	def nodeId():
		return	OpenMaya.MTypeId(0x6D746900)
		
	@staticmethod
	def nodeCreator():
		return	OpenMayaMPx.asMPxPtr( indigoText() )
	
	@staticmethod
	def nodeClassify():
		if version < 201100:
			return	"utility/color"
		else:
			return "utility/igo"
	
	def __init__(self ):
		OpenMayaMPx.MPxNode.__init__(self)
		
	@staticmethod
	def nodeInitializer():
		
		nAttr			= OpenMaya.MFnNumericAttribute()
		lAttr			= OpenMaya.MFnLightDataAttribute()
		enumAttr		= OpenMaya.MFnEnumAttribute()
		tAttr			= OpenMaya.MFnTypedAttribute()
		gAttr			= OpenMaya.MFnGenericAttribute()
		sAttr			= OpenMaya.MFnStringData()
		
		text = OpenMaya.MObject()
		mode = OpenMaya.MObject()
		outColor = OpenMaya.MObject()
		outAlpha = OpenMaya.MObject()
		
		kDefaultStringAttrValue = "default"		
		stringData = sAttr.create(kDefaultStringAttrValue)

		try:
			indigoText.text = tAttr.create("text", "t", OpenMaya.MFnData.kString, stringData)
			tAttr.setKeyable(0)
			tAttr.setStorable(1)
			tAttr.setReadable(1)
			tAttr.setWritable(1)
			
			# this attribute is not exposed
			indigoText.mode = enumAttr.create("mode",	"m",	0)
			enumAttr.addField( "Shader", 0)
			enumAttr.addField( "Link", 1)
			enumAttr.addField( "XML", 2)
			enumAttr.setKeyable(0)
			enumAttr.setStorable(0)
			enumAttr.setReadable(0)
			enumAttr.setWritable(0)
			enumAttr.setHidden(1)
	
			indigoText.outColor	= nAttr.createColor("outColor", "oc")
			nAttr.setKeyable(0)
			nAttr.setStorable(0)
			nAttr.setReadable(1)
			nAttr.setWritable(0)
				
			indigoText.outAlpha = nAttr.create("outAlpha", "oa",	OpenMaya.MFnNumericData.kFloat)
			nAttr.setKeyable(0)
			nAttr.setStorable(0)
			nAttr.setReadable(1)
			nAttr.setWritable(0)
		
		except:
			sys.stderr.write("Failed to create attributes in textFileNode\n")
			raise

		try:
			indigoText.addAttribute(indigoText.mode)
			indigoText.addAttribute(indigoText.text)
			indigoText.addAttribute(indigoText.outColor)
			indigoText.addAttribute(indigoText.outAlpha)
		
		except:
			sys.stderr.write("Failed to add attributes\n")
			raise
		
#-------------------------------------------------------------------------------

#class	indigoLink(OpenMayaMPx.MPxNode):
#	
#	@staticmethod
#	def nodeName():
#		return	"indigoLink"
#	
#	@staticmethod
#	def nodeId():
#		return	OpenMaya.MTypeId(0x6D746932)
#		
#	@staticmethod
#	def nodeCreator():
#		return	OpenMayaMPx.asMPxPtr( indigoLink() )
#	
#	@staticmethod
#	def nodeClassify():
#		return	"utility/general"
#	
#	def __init__(self ):
#		OpenMayaMPx.MPxNode.__init__(self)
#		
#	@staticmethod
#	def nodeInitializer():
#		
#		nAttr			= OpenMaya.MFnNumericAttribute()
#		lAttr			= OpenMaya.MFnLightDataAttribute()
#		enumAttr		= OpenMaya.MFnEnumAttribute()
#		tAttr			= OpenMaya.MFnTypedAttribute()
#		gAttr			= OpenMaya.MFnGenericAttribute()
#		sAttr			= OpenMaya.MFnStringData()
#		
#		kDefaultStringAttrValue = "default"		
#		defaultPath = sAttr.create()
#		path = OpenMaya.MObject()
#		outColor = OpenMaya.MObject()
#		outAlpha = OpenMaya.MObject()
#
#		try:
#			indigoLink.path = tAttr.create("path", "p", OpenMaya.MFnData.kString, defaultPath)
#			tAttr.setKeyable(0)
#			tAttr.setStorable(0)
#			tAttr.setReadable(1)
#			tAttr.setWritable(1)
#	
#			indigoLink.outColor	= nAttr.createColor("outColor", "oc")
#			nAttr.setKeyable(0)
#			nAttr.setStorable(0)
#			nAttr.setReadable(1)
#			nAttr.setWritable(0)
#				
#			indigoLink.outAlpha = nAttr.create("outAlpha", "oa",	OpenMaya.MFnNumericData.kFloat)
#			nAttr.setKeyable(0)
#			nAttr.setStorable(0)
#			nAttr.setReadable(1)
#			nAttr.setWritable(0)
##			nAttr.setDefault(	0.2 )
#		
#		except:
#			sys.stderr.write("Failed to create attributes in textFileNode\n")
#			raise
#
#		try:
#			indigoLink.addAttribute(indigoLink.path)
#			indigoLink.addAttribute(indigoLink.outColor)
#			indigoLink.addAttribute(indigoLink.outAlpha)
#		
#		except:
#			sys.stderr.write("Failed to add attributes\n")
#			raise
			
# ------------------------------------------------------------------------------

class	indigoShader(OpenMayaMPx.MPxNode):

	@staticmethod
	def nodeName():
		return	"indigoShader"

	@staticmethod
	def nodeId():
		return	OpenMaya.MTypeId(0x6D746901)
		
	@staticmethod
	def nodeCreator():
		return	OpenMayaMPx.asMPxPtr( indigoShader() )
				
	@staticmethod
	def nodeClassify():
		if version < 201100:
			return	"shader/surface"
		else:
			return "shader/surface/igo"

	def __init__(self):
		OpenMayaMPx.MPxNode.__init__(self)
		
		iMaterialType											= OpenMaya.MObject()
		aOutColor												= OpenMaya.MObject()
		aOutTransparency										= OpenMaya.MObject()
#		aOutNormal											= OpenMaya.MObject()
		
		# Common attributes
		common_Colour										= OpenMaya.MObject()
		common_BumpMapping								= OpenMaya.MObject()
		common_Layer											= OpenMaya.MObject()
		baseEmission											= OpenMaya.MObject()
		emission												= OpenMaya.MObject()
		common_Transparency								= OpenMaya.MObject()
		
		# Diffuse	material
		diffuse_Transmitter									= OpenMaya.MObject()
		aRandomTriCol										= OpenMaya.MObject()
		
		# Oren-Nayar	material
		orenNayar_sigma										= OpenMaya.MObject()
		
		# Phong material
		phong_IOR												= OpenMaya.MObject()
		phong_Exponent										= OpenMaya.MObject()
		phong_SpecularReflectivity							= OpenMaya.MObject()
		
		# Specular
		specular_Transparent									= OpenMaya.MObject()
		aUseAbsorptionLayerTransmittance					= OpenMaya.MObject()
		absorption_layer_transmittance						= OpenMaya.MObject()
		aSpecularInternalMedium								= OpenMaya.MObject()
		
		# Glossy Transparent
		glossyTransparent_Exponent							= OpenMaya.MObject()
		
		
	def compute(self, plug, block):

#			if	(plug	== aOutColor
#				|| plug	==	aOutColorR
#				|| plug	==	aOutColorG
#				|| plug	==	aOutColorB
#				|| plug	==	aOutTransparency
#				|| plug	==	aOutTransR
#				|| plug	==	aOutTransG
#				|| plug	==	aOutTransB )
	
		if plug == self.aOutColor: # or plug.parent()	==	self.aOutColor:
				
				resultColor	= block.inputValue( self.common_Colour	).asFloatVector()
				outColorHandle	= block.outputValue(	self.aOutColor	)
				outColorHandle.setMFloatVector(resultColor)
				outColorHandle.setClean()
				
#		elif plug == self.aOutTransparency:
#				
#				transparent	= block.inputValue( self.specular_Transparent ).asBool()
#				outTransValue = OpenMaya.MFloatVector(0.0, 0.0, 0.0)
#				
#				outTransHandle	= block.outputValue(	self.aOutTransparency	)
#				
#				if transparent == True:
#					outTransValue = (0.9, 0.9, 0.9)
##				else:
##					outTransValue = (0.0, 0.0, 0.0)
#					
#				outTransHandle.setMFloatVector(outTransValue)
#				outTransHandle.setClean()
				
		else:
				return OpenMaya.kUnknownParameter	
	
	@staticmethod
	def nodeInitializer():
		nAttr			= OpenMaya.MFnNumericAttribute()
		lAttr			= OpenMaya.MFnLightDataAttribute()
		enumAttr		= OpenMaya.MFnEnumAttribute()
		tAttr			= OpenMaya.MFnTypedAttribute()
		gAttr			= OpenMaya.MFnGenericAttribute()
	
		try:
		
				indigoShader.iMaterialType	= enumAttr.create("material",	"mat",	0)
				enumAttr.addField( "Diffuse", 0)
				enumAttr.addField( "Phong", 1)
				enumAttr.addField( "Oren-Nayar", 2)
				enumAttr.addField( "Specular", 3)
				enumAttr.addField( "Glossy Transparent", 4)
				enumAttr.addField( "Null", 5)
				#enumAttr.addField( "Blend", n)
	
				indigoShader.aOutColor	= nAttr.createColor("outColor", "oc")
				nAttr.setKeyable(0)
				nAttr.setStorable(0)
				nAttr.setReadable(1)
				nAttr.setWritable(0)
	
				indigoShader.aOutTransparency	=	nAttr.createColor("outTransparency",	"ot")
				nAttr.setKeyable(0)
				nAttr.setStorable(0)
				nAttr.setReadable(1)
				nAttr.setWritable(0)
	
#				indigoShader.aOutNormal	=	nAttr.createPoint("outNormal", "on")
#				nAttr.setKeyable(0)
#				nAttr.setStorable(0)
#				nAttr.setReadable(1)
#				nAttr.setWritable(0)
				
				# Common	attributes
				indigoShader.common_Colour	= nAttr.createColor("color", "c")
				makeInput( nAttr )
				nAttr.setUsedAsColor(1)
				nAttr.setDefault(0.8, 0.8,	0.8)
				#-	normalCamera hidden,	allows bump	mapping
				indigoShader.common_BumpMapping = nAttr.createPoint("normalCamera","n")
				makeInput( nAttr )
				nAttr.setHidden(1)
				nAttr.setDefault(0, 0, 1.0)
				#-
 #				indigoShader.common_BumpMapABC = nAttr.createColor("bumpABC",	"babc")
 #				makeInput(	nAttr	)
 #				nAttr.setUsedAsColor(0)
	#			nAttr.setDefault(0.0, 1.0, 0.0)
				#-
#				indigoShader.common_BumpMapExp = nAttr.create("bumpExponent",	"be",	OpenMaya.MFnNumericData.kFloat)
#				makeInput(	nAttr	)
#				nAttr.setDefault( 1.0 )
				#-
				indigoShader.common_Transparency	= nAttr.createColor("transparency","t")
				makeInput( nAttr )
				nAttr.setDefault(0.0, 0.0,	0.0)
				#-
				indigoShader.common_Layer = nAttr.create("lightLayer", "ll", OpenMaya.MFnNumericData.kInt)
				makeInput( nAttr )
				#-
				indigoShader.baseEmission = nAttr.createColor("ambientColor", "bem")
				makeInput( nAttr )
				nAttr.setUsedAsColor(1)
				nAttr.setDefault(0.0, 0.0,	0.0)
				#-
				indigoShader.emission =	nAttr.createColor("incandescence", "em")
				makeInput( nAttr )
				nAttr.setUsedAsColor(1)
				nAttr.setDefault(0.0, 0.0,	0.0)
				
				# Diffuse Material
				indigoShader.diffuse_Transmitter	= nAttr.create("diffuseTransmitter", "trs", OpenMaya.MFnNumericData.kBoolean)
				makeInput( nAttr )
				indigoShader.aRandomTriCol	= nAttr.create("randomTriangles", "rnt", OpenMaya.MFnNumericData.kBoolean)
				makeInput( nAttr )
				
				# Oren-Nayar Material
				indigoShader.orenNayar_sigma = nAttr.create("orenNayarSigma", "ons",	OpenMaya.MFnNumericData.kFloat)
				makeInput( nAttr )
				nAttr.setDefault(	0.2 )
				
				# Phong Material
				#-
				indigoShader.phong_Type	= enumAttr.create("specType",	"spt",	0)
				enumAttr.addField( "ior",						0)
				enumAttr.addField( "specular reflectivity",		1)
				enumAttr.addField( "nkData",					2)
				
				indigoShader.phong_IOR = nAttr.create("phongIOR", "pior", OpenMaya.MFnNumericData.kFloat)
				makeInput( nAttr )
				nAttr.setDefault(	1.334 )
				#-
				indigoShader.phong_Exponent =	nAttr.create("cosinePower", "pe", OpenMaya.MFnNumericData.kFloat)
				makeInput( nAttr )
				nAttr.setDefault(	800.0 )
				#-
				indigoShader.phong_SpecularReflectivity =	nAttr.createColor("phongSpecularReflectivity", "psrs")
				makeInput( nAttr )
				nAttr.setUsedAsColor(1)
				nAttr.setDefault(0.0, 0.0, 0.0)
				
				# Specular
				indigoShader.specular_Transparent =	nAttr.create("specularTransparent", "st", OpenMaya.MFnNumericData.kBoolean, 1)
				makeInput( nAttr )
				indigoShader.aUseAbsorptionLayerTransmittance =	nAttr.create("useTransmittanceLayer", "utl", OpenMaya.MFnNumericData.kBoolean)
				makeInput( nAttr )
				indigoShader.absorption_layer_transmittance = nAttr.createColor("transmittanceLayer", "tl")
				indigoShader.aSpecularInternalMedium = nAttr.createColor("internalMedium", "inm")
				
				# Glossy	Transparent
				indigoShader.glossyTransparent_Exponent =	nAttr.create("glossyTransparentExponent", "ge", OpenMaya.MFnNumericData.kFloat)
				makeInput( nAttr )
				nAttr.setDefault(800.0)
				
		except:
				sys.stderr.write("Failed to create attributes\n")
				raise
				
		try:
				indigoShader.addAttribute(indigoShader.iMaterialType)		
				indigoShader.addAttribute(indigoShader.aOutColor)
				indigoShader.addAttribute(indigoShader.aOutTransparency)
				
				#Common attrs
				indigoShader.addAttribute(indigoShader.common_Colour)
				indigoShader.addAttribute(indigoShader.common_BumpMapping)
				indigoShader.addAttribute(indigoShader.common_Transparency)
				indigoShader.addAttribute(indigoShader.common_Layer)
				indigoShader.addAttribute(indigoShader.baseEmission)
				indigoShader.addAttribute(indigoShader.emission)
				
				#Diffuse	Material
				indigoShader.addAttribute(indigoShader.diffuse_Transmitter)
				indigoShader.addAttribute(indigoShader.aRandomTriCol)
				
				#Oren-Nayar	Material
				indigoShader.addAttribute(indigoShader.orenNayar_sigma)
				
				#Phong Material
				indigoShader.addAttribute(indigoShader.phong_Type)
				indigoShader.addAttribute(indigoShader.phong_IOR)
				indigoShader.addAttribute(indigoShader.phong_Exponent)
				indigoShader.addAttribute(indigoShader.phong_SpecularReflectivity)
				
#				# Medium
#				indigoShader.addAttribute(indigoShader.medium_Precedence)
#				indigoShader.addAttribute(indigoShader.medium_Type)
#				indigoShader.addAttribute(indigoShader.medium_IOR)
#				indigoShader.addAttribute(indigoShader.medium_CauchyB)
#				indigoShader.addAttribute(indigoShader.medium_Density)
#				indigoShader.addAttribute(indigoShader.medium_SubsurfaceScattering)
#				indigoShader.addAttribute(indigoShader.medium_SubsurfaceScatteringCoefficientSpectrum)
#				indigoShader.addAttribute(indigoShader.medium_SubsurfaceScatteringPhaseFunction)
#				indigoShader.addAttribute(indigoShader.medium_SubsurfaceScatteringGSpectrum)
#				indigoShader.addAttribute(indigoShader.medium_MelaninFraction)
#				indigoShader.addAttribute(indigoShader.medium_MelaninTypeBlend)
#				indigoShader.addAttribute(indigoShader.medium_HemoglobinFraction)
				
				# Specular
				indigoShader.addAttribute(indigoShader.specular_Transparent)
				indigoShader.addAttribute(indigoShader.aUseAbsorptionLayerTransmittance)
				indigoShader.addAttribute(indigoShader.absorption_layer_transmittance)
				indigoShader.addAttribute(indigoShader.aSpecularInternalMedium)
				
				# Glossy	Transparent
				indigoShader.addAttribute(indigoShader.glossyTransparent_Exponent)
				
		except:
				sys.stderr.write("Failed to add attributes\n")
				raise
	
		try:
				# Diffuse Material
				indigoShader.attributeAffects	(indigoShader.common_Colour,					indigoShader.aOutColor)
				indigoShader.attributeAffects	(indigoShader.common_BumpMapping,			indigoShader.aOutColor)
				indigoShader.attributeAffects	(indigoShader.common_Transparency,			indigoShader.aOutTransparency)
				indigoShader.attributeAffects	(indigoShader.specular_Transparent,				indigoShader.aOutTransparency)
				indigoShader.attributeAffects	(indigoShader.absorption_layer_transmittance,	indigoShader.aOutTransparency)
				
		except:
				sys.stderr.write("Failed in setting attributeAffects\n")
				raise
			
# ------------------------------------------------------------------------------

#class	indigoSpecular(OpenMayaMPx.MPxNode):
#
#	@staticmethod
#	def nodeName():
#		return	"indigoSpecular"
#
#	@staticmethod
#	def nodeId():
#		return	OpenMaya.MTypeId(0x6D746903)
#		
#	@staticmethod
#	def nodeCreator():
#		return	OpenMayaMPx.asMPxPtr( indigoSpecular() )
#				
#	@staticmethod
#	def nodeClassify():
#		return	"shader/surface/igo"
#
#	def __init__(self):
#		OpenMayaMPx.MPxNode.__init__(self)
#		
#		aOutColor					= OpenMaya.MObject()
#		aOutTransparency			= OpenMaya.MObject()
#		
#		# Common attributes
#		aNormalCamera			= OpenMaya.MObject()
#		aBaseEmission			= OpenMaya.MObject()
#		aEmission					= OpenMaya.MObject()
#		aLayer						= OpenMaya.MObject()
#		aTransparency				= OpenMaya.MObject()
#		
#		# Specular
#		aTransparent				= OpenMaya.MObject()
#		aUseAbsorptionLayer		= OpenMaya.MObject()
#		aAbsorptionLayer			= OpenMaya.MObject()
#		aMedium					= OpenMaya.MObject()
#		aIor						= OpenMaya.MObject()
#		
#		
##	def compute(self, plug, block):
##
###			if	(plug	== aOutColor
###				|| plug	==	aOutColorR
###				|| plug	==	aOutColorG
###				|| plug	==	aOutColorB
###				|| plug	==	aOutTransparency
###				|| plug	==	aOutTransR
###				|| plug	==	aOutTransG
###				|| plug	==	aOutTransB )
##	
##		if plug == self.aOutColor or plug.parent()	==	self.aOutColor:
##				
##				resultColor	= block.inputValue( self.common_Colour	).asFloatVector()
##				
##				
##				outColorHandle	= block.outputValue(	self.aOutColor	)
##				outColorHandle.setMFloatVector(resultColor)
##				outColorHandle.setClean()
##		else:
##				return OpenMaya.kUnknownParameter	
#	
#	@staticmethod
#	def nodeInitializer():
#		nAttr			= OpenMaya.MFnNumericAttribute()
#		lAttr			= OpenMaya.MFnLightDataAttribute()
#		enumAttr		= OpenMaya.MFnEnumAttribute()
#		tAttr			= OpenMaya.MFnTypedAttribute()
#		gAttr			= OpenMaya.MFnGenericAttribute()
#	
#		try:
#	
#				indigoSpecular.aOutColor	= nAttr.createColor("outColor", "oc")
#				nAttr.setKeyable(0)
#				nAttr.setStorable(0)
#				nAttr.setReadable(1)
#				nAttr.setWritable(0)
#	
#				indigoSpecular.aOutTransparency	=	nAttr.createColor("outTransparency",	"ot")
#				nAttr.setKeyable(0)
#				nAttr.setStorable(0)
#				nAttr.setReadable(1)
#				nAttr.setWritable(0)
#				
##				# Common	attributes
##				indigoSpecular.common_Colour	= nAttr.createColor("color", "c")
##				makeInput( nAttr )
##				nAttr.setUsedAsColor(1)
##				nAttr.setDefault(0.8, 0.8,	0.8)
#
#				#-	normalCamera hidden,	allows bump	mapping
#				indigoSpecular.aNormalCamera = nAttr.createPoint("normalCamera","n")
#				makeInput( nAttr )
#				nAttr.setHidden(1)
##				nAttr.setDefault(0, 0, 1.0)
#				#-
#				indigoSpecular.aUseAbsorptionLayer	= nAttr.create("useAbsorptionLayer", "ual", OpenMaya.MFnNumericData.kBoolean)
##				makeInput( nAttr )
##				nAttr.setDefault(False)
#				#-
#				indigoSpecular.aAbsorptionLayer	= nAttr.createColor("absorptionLayer", "abl")
#				#-
#				indigoSpecular.aTransparent	= nAttr.create("transparent", "trs", OpenMaya.MFnNumericData.kBoolean)
#				#-
#				indigoSpecular.aTransparency	= nAttr.createColor("transparency","t")
#				makeInput( nAttr )
#				nAttr.setDefault(0.0, 0.0, 0.0)
#				#-
#				indigoSpecular.aLayer = nAttr.create("lightLayer", "ll", OpenMaya.MFnNumericData.kInt)
##				makeInput( nAttr )
#				#-
#				indigoSpecular.aIor = nAttr.create("ior", "ior", OpenMaya.MFnNumericData.kFloat)
#				nAttr.setDefault(1.334)
#				makeInput( nAttr )
##				makeInput( nAttr )
#				#-
#				indigoSpecular.aBaseEmission = nAttr.createColor("ambientColor", "bem")
#				makeInput( nAttr )
#				nAttr.setUsedAsColor(1)
#				nAttr.setDefault(0.0, 0.0,	0.0)
#				#-
#				indigoSpecular.aEmission =	nAttr.createColor("incandescence", "em")
#				makeInput( nAttr )
#				nAttr.setUsedAsColor(1)
#				nAttr.setDefault(0.0, 0.0,	0.0)
#				#-
#				indigoSpecular.aMedium =	nAttr.createColor("internalMedium", "inm")
#				makeInput( nAttr )
#				nAttr.setUsedAsColor(1)
#				nAttr.setDefault(0.0, 0.0, 0.0)
#	
#				
#		except:
#				sys.stderr.write("Failed to create attributes\n")
#				raise
#				
#		try:	
#				indigoSpecular.addAttribute(indigoSpecular.aOutColor)
#				indigoSpecular.addAttribute(indigoSpecular.aOutTransparency)
#				
#				#Common attrs
#				indigoSpecular.addAttribute(indigoSpecular.aNormalCamera)
#				indigoSpecular.addAttribute(indigoSpecular.aTransparency)
#				indigoSpecular.addAttribute(indigoSpecular.aLayer)
#				indigoSpecular.addAttribute(indigoSpecular.aBaseEmission)
#				indigoSpecular.addAttribute(indigoSpecular.aEmission)
#				
#				# Specular
#				indigoSpecular.addAttribute(indigoSpecular.aTransparent)
#				indigoSpecular.addAttribute(indigoSpecular.aIor)
#				indigoSpecular.addAttribute(indigoSpecular.aMedium)
#				indigoSpecular.addAttribute(indigoSpecular.aUseAbsorptionLayer)
#				indigoSpecular.addAttribute(indigoSpecular.aAbsorptionLayer)
#				
#		except:
#				sys.stderr.write("Failed to add attributes\n")
#				raise
	
#		try:
#				# Diffuse Material
#				indigoSpecular.attributeAffects	(indigoSpecular.common_Colour,			indigoSpecular.aOutColor)
#				indigoSpecular.attributeAffects	(indigoSpecular.absorbtion,	indigoSpecular.aOutTransparency)
##				indigoSpecular.attributeAffects	(indigoSpecular.common_BumpMapping,	indigoSpecular.aOutNormal)
#				
#		except:
#				sys.stderr.write("Failed in setting attributeAffects\n")
#				raise
		
def makeInput(attr):
	attr.setKeyable(1)
	
def makeOutput(attr):
	attr.setStorable(0)
	attr.setWritable(0)
